@@ -1,4 +1,4 @@
-/* Lightweight thread library for Objective Caml
+/* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Module Lwt_unix_stubs
  * Copyright (C) 2009-2010 Jérémie Dimino
@@ -22,11 +22,6 @@
 
 #include "lwt_config.h"
 
-#if defined(LWT_ON_WINDOWS)
-#  include <winsock2.h>
-#  include <windows.h>
-#endif
-
 #define _GNU_SOURCE
 #define _POSIX_PTHREAD_SEMANTICS
 
@@ -140,7 +135,7 @@ void lwt_unix_not_available(char const *feature)
    | Operation on bigarrays                                          |
    +-----------------------------------------------------------------+ */
 
-CAMLprim value lwt_unix_blit_bytes_bytes(value val_buf1, value val_ofs1, value val_buf2, value val_ofs2, value val_len)
+CAMLprim value lwt_unix_blit(value val_buf1, value val_ofs1, value val_buf2, value val_ofs2, value val_len)
 {
   memmove((char*)Caml_ba_data_val(val_buf2) + Long_val(val_ofs2),
          (char*)Caml_ba_data_val(val_buf1) + Long_val(val_ofs1),
@@ -148,7 +143,7 @@ CAMLprim value lwt_unix_blit_bytes_bytes(value val_buf1, value val_ofs1, value v
   return Val_unit;
 }
 
-CAMLprim value lwt_unix_blit_string_bytes(value val_buf1, value val_ofs1, value val_buf2, value val_ofs2, value val_len)
+CAMLprim value lwt_unix_blit_from_bytes(value val_buf1, value val_ofs1, value val_buf2, value val_ofs2, value val_len)
 {
   memcpy((char*)Caml_ba_data_val(val_buf2) + Long_val(val_ofs2),
          String_val(val_buf1) + Long_val(val_ofs1),
@@ -156,7 +151,7 @@ CAMLprim value lwt_unix_blit_string_bytes(value val_buf1, value val_ofs1, value
   return Val_unit;
 }
 
-CAMLprim value lwt_unix_blit_bytes_string(value val_buf1, value val_ofs1, value val_buf2, value val_ofs2, value val_len)
+CAMLprim value lwt_unix_blit_to_bytes(value val_buf1, value val_ofs1, value val_buf2, value val_ofs2, value val_len)
 {
   memcpy(String_val(val_buf2) + Long_val(val_ofs2),
          (char*)Caml_ba_data_val(val_buf1) + Long_val(val_ofs1),
@@ -560,9 +555,11 @@ void lwt_unix_send_notification(int id)
 #if defined(LWT_ON_WINDOWS)
     if (ret == SOCKET_ERROR) {
       error = WSAGetLastError();
-      lwt_unix_mutex_unlock(&notification_mutex);
-      win32_maperr(error);
-      uerror("send_notification", Nothing);
+      if (error != WSANOTINITIALISED) {
+	lwt_unix_mutex_unlock(&notification_mutex);
+	win32_maperr(error);
+	uerror("send_notification", Nothing);
+      } /* else we're probably shutting down, so ignore the error */
     }
 #else
     if (ret < 0) {
@@ -587,7 +584,7 @@ value lwt_unix_send_notification_stub(value id)
 
 value lwt_unix_recv_notifications()
 {
-  int ret, i;
+  int ret, i, current_index;
   value result;
 #if !defined(LWT_ON_WINDOWS)
   sigset_t new_mask;
@@ -616,8 +613,23 @@ value lwt_unix_recv_notifications()
     unix_error(error, "recv_notifications", Nothing);
   }
 #endif
+
+  do {
+    /*
+     release the mutex while calling caml_alloc,
+     which may call gc and switch the thread,
+     resulting in a classical deadlock,
+     when thread in question tries another send
+    */
+    current_index = notification_index;
+    lwt_unix_mutex_unlock(&notification_mutex);
+    result = caml_alloc_tuple(current_index);
+    lwt_unix_mutex_lock(&notification_mutex);
+    /* check that no new notifications appeared meanwhile (rare) */
+  }
+  while (current_index != notification_index);
+
   /* Read all pending notifications. */
-  result = caml_alloc_tuple(notification_index);
   for (i = 0; i < notification_index; i++)
     Field(result, i) = Val_long(notifications[i]);
   /* Reset the index. */
@@ -648,7 +660,7 @@ static SOCKET socket_r, socket_w;
 
 static int windows_notification_send()
 {
-  char buf;
+  char buf = '!';
   return send(socket_w, &buf, 1, 0);
 }
 
@@ -881,7 +893,7 @@ CAMLprim value lwt_unix_remove_signal(value val_signum)
 }
 
 /* Mark all signals as non-monitored. */
-CAMLprim value lwt_unix_init_signals()
+CAMLprim value lwt_unix_init_signals(value Unit)
 {
   int i;
   for (i = 0; i < NSIG; i++)
@@ -1309,6 +1321,7 @@ CAMLprim value lwt_unix_start_job(value val_job, value val_async_method)
   struct stack_frame *node;
 #endif
   lwt_unix_async_method async_method = Int_val(val_async_method);
+  int done = 0;
 
   /* Fallback to synchronous call if there is no worker available and
      we can not launch more threads. */
@@ -1354,7 +1367,16 @@ CAMLprim value lwt_unix_start_job(value val_job, value val_async_method)
       lwt_unix_condition_signal(&pool_condition);
       lwt_unix_mutex_unlock(&pool_mutex);
     }
-    return Val_bool(job->state == LWT_UNIX_JOB_STATE_DONE);
+
+    done = job->state == LWT_UNIX_JOB_STATE_DONE;
+    if (done) {
+      /* Wait for the mutex to be released because the job is going to
+         be freed immediately. */
+      lwt_unix_mutex_lock(&job->mutex);
+      lwt_unix_mutex_unlock(&job->mutex);
+    }
+
+    return Val_bool(done);
 
   case LWT_UNIX_ASYNC_METHOD_SWITCH:
 #if defined(LWT_UNIX_SIGNAL_ASYNC_SWITCH)
@@ -1418,7 +1440,14 @@ CAMLprim value lwt_unix_start_job(value val_job, value val_async_method)
 
       /* This thread is now running caml code. */
       //caml_c_thread_register();
-      return Val_bool(job->state == LWT_UNIX_JOB_STATE_DONE);
+
+      done = job->state == LWT_UNIX_JOB_STATE_DONE;
+      if (done) {
+        lwt_unix_mutex_lock(&job->mutex);
+        lwt_unix_mutex_unlock(&job->mutex);
+      }
+
+      return Val_bool(done);
     }
 
 #else /* defined(LWT_UNIX_SIGNAL_ASYNC_SWITCH) */
@@ -1476,7 +1505,7 @@ CAMLprim value lwt_unix_run_job_sync(value val_job)
   return job->result(job);
 }
 
-CAMLprim value lwt_unix_reset_after_fork()
+CAMLprim value lwt_unix_reset_after_fork(value Unit)
 {
   if (threading_initialized) {
 #if defined(LWT_UNIX_HAVE_ASYNC_SWITCH)
@@ -1501,7 +1530,7 @@ CAMLprim value lwt_unix_reset_after_fork()
    | Statistics and control                                          |
    +-----------------------------------------------------------------+ */
 
-CAMLprim value lwt_unix_pool_size()
+CAMLprim value lwt_unix_pool_size(value Unit)
 {
   return Val_int(pool_size);
 }
@@ -1512,12 +1541,12 @@ CAMLprim value lwt_unix_set_pool_size(value val_size)
   return Val_unit;
 }
 
-CAMLprim value lwt_unix_thread_count()
+CAMLprim value lwt_unix_thread_count(value Unit)
 {
   return Val_int(thread_count);
 }
 
-CAMLprim value lwt_unix_thread_waiting_count()
+CAMLprim value lwt_unix_thread_waiting_count(value Unit)
 {
   return Val_int(thread_waiting_count);
 }
