@@ -1,4 +1,4 @@
-(* Lightweight thread library for Objective Caml
+(* Lightweight thread library for OCaml
  * http://www.ocsigen.org/lwt
  * Module Test_lwt_io
  * Copyright (C) 2009 Jérémie Dimino
@@ -29,34 +29,34 @@ let suite = suite "lwt_io" [
     (fun () ->
        let sent = ref [] in
        let oc = Lwt_io.make ~mode:output (fun buf ofs len ->
-                                            let str = String.create len in
-                                            Lwt_bytes.blit_bytes_string buf ofs str 0 len;
-                                            sent := str :: !sent;
+                                            let bytes = Bytes.create len in
+                                            Lwt_bytes.blit_to_bytes buf ofs bytes 0 len;
+                                            sent := bytes :: !sent;
                                             return len) in
-       lwt () = write oc "foo" in
-       lwt () = write oc "bar" in
+       write oc "foo" >>= fun () ->
+       write oc "bar" >>= fun () ->
        if !sent <> [] then
          return false
        else
-         lwt () = Lwt_unix.yield () in
-         return (!sent = ["foobar"]));
+         Lwt_unix.yield () >>= fun () ->
+         return (!sent = [Bytes.of_string "foobar"]));
 
   test "auto-flush in atomic"
     (fun () ->
        let sent = ref [] in
        let oc = make ~mode:output (fun buf ofs len ->
-                                     let str = String.create len in
-                                     Lwt_bytes.blit_bytes_string buf ofs str 0 len;
-                                     sent := str :: !sent;
+                                     let bytes = Bytes.create len in
+                                     Lwt_bytes.blit_to_bytes buf ofs bytes 0 len;
+                                     sent := bytes :: !sent;
                                      return len) in
        atomic
          (fun oc ->
-            lwt () = write oc "foo" in
-            lwt () = write oc "bar" in
+            write oc "foo" >>= fun () ->
+            write oc "bar" >>= fun () ->
             if !sent <> [] then
               return false
             else
-              lwt () = Lwt_unix.yield () in
-              return (!sent = ["foobar"]))
+              Lwt_unix.yield () >>= fun () ->
+              return (!sent = [Bytes.of_string "foobar"]))
          oc);
 ]
